import setuptools

setuptools.setup(
    name="libgpac",
    version="2.2",
    author="Jean Le Feuvre",
    author_email="trac-gpac@listes.telecom-paristech.fr",
    description="Python ctypes bindings for libgpac",
    url="https://gpac.io",
    packages=['libgpac'],
    install_requires=[],
    classifiers=[
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)",
    "Operating System :: OS Independent",
],

)
