
my_ext = {
    whoami: "GPAC-GUI-Extension-DIS",
    name: "DASH-IF sequences",
    icon: "dashif.svg",
    author: "JeanLF",
    description: "DASH-IF test sequences",
    url: "http://dashif.org/",
    execjs: ["dashif_tests.js"],
    autostart: false,
    config_id: "TDashIF",
    requires_gl: false,
    version_major: 1,
    version_minor: 0
};

