/* Automatically generated by configure */
#ifndef GF_CONFIG_H
#define GF_CONFIG_H
#define GPAC_CONFIGURATION "--prefix=/usr --mandir=/usr/share/man --disable-oss-audio --disable-dvb4linux --disable-qjs --enable-pic"
#define GF_STATIC static
#define GF_NOT_EXPORTED
#define GPAC_HAS_MTIM_NSEC
#define GPAC_CONFIG_ARM
#define GPAC_CONFIG_LINUX
#define GPAC_SCHED_DEFAULT "lock"
#define GPAC_HAS_SSL
#define GPAC_HAS_POLL
#define GPAC_64_BITS
#define GPAC_HAS_JPEG
#define GPAC_HAS_PNG
#define GPAC_HAS_STRLCPY
#define GPAC_HAS_STRLCAT
#define GPAC_HAS_SOCK_UN
#define GPAC_HAS_IFADDRS
#define GPAC_HAS_LZMA
#define GPAC_HAS_JP2
#define GPAC_HAS_XVID
#define GPAC_HAS_VORBIS
#define GPAC_HAS_THEORA
#define GPAC_HAS_FFMPEG
#define FFMPEG_ENABLE_VVC
#define GPAC_HAS_HTTP2
#define GPAC_HAS_CURL
#define GPAC_DISABLE_3D
#define GPAC_HAS_IPV6
#endif
